program aladin2udcf;

(*           Autor : Kai Schrder
             Datum : 17.11.1999
   letzte nderung : 28. 1.2000

   Version 0.0.8

   Konvertiert Aladin-Logbuchdateien in das UDCF-Format (nur die TGs mit
   Profildaten). Ohne Kommandozeilen-Parameter in deutsch, mit Parameter
   "-e" englischer Text.

   Converts Aladin-logbook files into UDCF (only dives with profile data).
   Without commandline parameter German version, with "-e" parameter
   English texts.
*)

  uses
    crt, dos;

  const
    version = '0.0.7';

  var
    nn, i, jahr, monat, tag, std, min, intervall : integer;
    year, month, day, dow, hour, minute, sek, hun : word;
    ort : string;
    j, anz_tg_in_datei : byte;
    b : array [0..255] of byte;
    dateiname, inputdatei, outputdatei : string;
    logdat : file of char;
    udcfdat : text;
    vorheriger_tg_mit_grafik, infinity, erste_tiefen_bytes, deutsch : boolean;
    hoehe, tankvolume, temperature, p_end, tiefe : real;


  procedure lese_1_byte (var ein_byte : byte);

    var c : char;

    begin
      read (logdat, c);
      ein_byte := ord (c)
    end; (* lese_1_byte *)

  procedure dateikopf_lesen;

    begin
      for i := 0 to 11 do lese_1_byte (b [i]);
      anz_tg_in_datei := b [5] * 256 + b [6];
      if deutsch
        then
          writeln (' In der Datei sind ', anz_tg_in_datei, ' TGs gespeichert.')
        else
          writeln (' In this file ', anz_tg_in_datei, ' dives are stored.');
      vorheriger_tg_mit_grafik := true
    end; (* dateikopf_lesen *)

  procedure datum;

    var
      x : real;

    begin
      if vorheriger_tg_mit_grafik then lese_1_byte (b [0]);
      lese_1_byte (b [1]);
      vorheriger_tg_mit_grafik := true;

      for i := 0 to 2 do lese_1_byte (b [i]);
      x := b [2] * 256.0 * 256.0 + b [1] * 256.0 + b [0];

      jahr := trunc (1600 + 4 * (x / (4 * 365.25)));
      x := x - (jahr - 1600) * 365.25 + trunc (x / (36525) + 1);

      monat := 1;
      if x > 31 then begin
                       monat := 2;
                       x := x - 31
                     end;
      if x > 28 then begin
                       monat := 3;
                       {Achtung Schaltjahr Sip 03.06.96}
                       x := x - 28
                     end;
      if x > 31 then begin
                       monat := 4;
                       x := x - 31
                     end;
      if x > 30 then begin
                       monat := 5;
                       x := x - 30
                     end;
      if x > 31 then begin
                       monat := 6;
                       x := x - 31
                     end;
      if x > 30 then begin
                       monat := 7;
                       x := x - 30
                     end;
      if x > 31 then begin
                       monat := 8;
                       x := x - 31
                     end;
      if x > 30 then begin
                       monat := 9;
                       x := x - 30
                     end;
      if x > 31 then begin
                       monat := 10;
                       x := x - 31
                     end;
      if x > 30 then begin
                       monat := 11;
                       x := x - 30
                     end;
      if x > 31 then begin
                       monat := 12;
                       x := x - 31
                     end;
      if x > 31 then writeln ('Fehler bei Datums-Entschlsselung ! / Error decoding date !');
      tag := trunc (x)
    end; (* datum *)

  procedure allgemeine_daten_lesen;

    begin

      datum;

      (* Zeitzone; 1 Byte *)
      lese_1_byte (b [0]);

      (* Uhrzeit, 2 Bytes *)
      lese_1_byte (b [0]);
      lese_1_byte (b [1]);
      std := (b [1] * 256 + b [0]) div 60;
      min := (b [1] * 256 + b [0]) mod 60;

      (* Ort, 1 + n Bytes *)
      ort := '';
      lese_1_byte (b [0]);  (* gibt direkt die Anzahl Zeichen des Ortes an *)
      for i := 1 to b [0] do begin
                               lese_1_byte (b [i]);
                               ort := ort + chr (b [i])
                             end;

      (* Tauchplatz, 1 + n Bytes *)
      lese_1_byte (b [0]);  (* gibt direkt die Anzahl Zeichen des Tauchplatzes an *)
      if b [0] >= 1 then ort := ort + ', ';
      for i := 1 to b [0] do begin
                               lese_1_byte (b [i]);
                               ort := ort + chr (b [i])
                             end;

      (* Hhenbereich, 1 Byte *)
      lese_1_byte (b [0]);
      if b [0] and 1 = 1 then hoehe := 0.0                (* 0m - 900m *)
      else if b [0] and 2 = 2 then hoehe := 900.0         (* 900m - 1750m *)
      else if b [0] and 4 = 4 then hoehe := 1750.0        (* 1750m - 2700m *)
      else if b [0] and 8 = 8 then hoehe := 2700.0        (* 2700m - 4000m *)
      else hoehe := 0.0;

      (* Intervall, 2 Bytes *)
      lese_1_byte (b [0]);
      lese_1_byte (b [1]);
      infinity := false;
      if b [0] < 255 then intervall := b [0] + b [1] * 256
                     else
                       (* b [0] = FFh : kein Intervall *)
                       infinity := true;

      (* Wetter, 1 Byte *)
      lese_1_byte (b [0]);

      (* Lufttemperatur, 2 Bytes *)
      lese_1_byte (b [0]);
      lese_1_byte (b [1]);

      (* Tauchanzug, 1 Byte *)
      lese_1_byte (b [0]);

      (* Flaschengre in Liter, 2 Bytes *)
      lese_1_byte (b [0]);
      lese_1_byte (b [1]);
      tankvolume := (b [0] + b [1] * 256) div 100;

      (* maximale Tiefe, 2 Bytes *)
      lese_1_byte (b [0]);
      lese_1_byte (b [1]);

      (* Tauchdauer, 2 Bytes *)
      lese_1_byte (b [0]);
      lese_1_byte (b [1]);

      (* minimale Wassertemperatur in Grad Celsius, 2 Bytes *)
      lese_1_byte (b [0]);
      lese_1_byte (b [1]);
      temperature := round ((b [0] + b [1] * 256) / 100);

      (* Luftverbrauch in bar, 2 Bytes *)
      lese_1_byte (b [0]);
      lese_1_byte (b [1]);
      p_end := 200.0 - (b [0] + b [1] * 256) div 100;

      (* Tauchgangstyp, 2 Bytes *)
      lese_1_byte (b [0]);
      lese_1_byte (b [1]);

      (* Aktivitten, 2 Bytes *)
      lese_1_byte (b [0]);
      lese_1_byte (b [1]);

      (* andere Aktivitten, 1 + n Bytes *)
      lese_1_byte (b [0]);
      for i := 1 to b [0] do lese_1_byte (b [i]);

      (* Tauchpartner, 1 + n Bytes *)
      lese_1_byte (b [0]);  (* gibt direkt die Anzahl Zeichen an *)
      for i := 1 to b [0] do lese_1_byte (b [i]);

      (* Kommentar, 1 + n Bytes *)
      lese_1_byte (b [0]);  (* gibt direkt die Anzahl Zeichen an *)
      for i := 1 to b [0] do lese_1_byte (b [i]);

      (* Alarmmeldungen, 2 Bytes *)
      lese_1_byte (b [0]);
      lese_1_byte (b [1]);

      (* TG-Zhler, 2 Bytes *)
      lese_1_byte (b [0]);
      lese_1_byte (b [1]);

      (* (noch) unbekannte 12 Bytes *)
      for i := 0 to 11 do lese_1_byte (b [i])
    end; (* allgemeine_daten_lesen *)

  procedure allgemeine_daten_schreiben;

    begin
      writeln (udcfdat, '  <dive>');
      writeln (udcfdat, '    <!-- dive ', j, ' out of ', anz_tg_in_datei, ' contained in ', inputdatei,' -->');
      writeln (udcfdat, '    <place>', ort, '</place>');
      writeln (udcfdat, '    <date><year>', jahr, '</year><month>', monat, '</month><day>', tag, '</day></date>');
      writeln (udcfdat, '    <time><hour>', std, '</hour><minute>', min, '</minute></time>');
      if infinity
        then
          writeln (udcfdat, '    <surfaceinterval>infinity</surfaceinterval>')
        else
          writeln (udcfdat, '    <surfaceinterval>', intervall, '.0</surfaceinterval>');
      writeln (udcfdat, '    <temperature>', temperature:3:1, '</temperature>');
      writeln (udcfdat, '    <density>1030.0</density>');
      writeln (udcfdat, '    <altitude>', hoehe:3:1, '</altitude>');
      writeln (udcfdat, '    <gases>');
      writeln (udcfdat, '      <mix>');
      writeln (udcfdat, '        <mixname>air</mixname>');
      writeln (udcfdat, '        <tank>');
      writeln (udcfdat, '          <pstart>20000000.0</pstart>');
      writeln (udcfdat, '          <pend>', ((200.0 - p_end) * 100000):10:1, '</pend>');
      writeln (udcfdat, '        </tank>');
      writeln (udcfdat, '        <o2>0.21</o2>');
      writeln (udcfdat, '        <n2>0.79</n2>');
      writeln (udcfdat, '        <he>0.0</he>');
      writeln (udcfdat, '      </mix>');
      writeln (udcfdat, '    </gases>');
    end; (* allgemeine_daten_schreiben *)

  procedure tg_kopfdaten_lesen_schreiben;

    begin
      writeln (udcfdat, '    <program>');
      writeln (udcfdat, '      <datatrak>');
      writeln (udcfdat, '        <tissues>8</tissues>');
      writeln (udcfdat, '        <!-- 2 bytes per tissue, not decoded yet -->');
      for i := 0 to 18 do lese_1_byte (b [i]);
      for i := 0 to 7 do
        writeln (udcfdat, '        <ip>', b [i * 2 + 1]:3, ' ', b [i * 2 + 2]:3, '</ip>');
      writeln (udcfdat, '      </datatrak>');
      writeln (udcfdat, '    </program>');
    end; (* tg_kopfdaten_lesen_schreiben *)

  procedure tg_daten_lesen_und_schreiben;

    var
      anz_tg_bytes, n : integer;

    begin
      (* TG-Daten-Zhler, 2 Bytes *)
      lese_1_byte (b [0]);
      lese_1_byte (b [1]);
      anz_tg_bytes := b [0] + b [1] * 256;


      if anz_tg_bytes > 0
        then
          (* Profildaten sind vorhanden, *)
          (* TG kann also in UDCF-Datei bernommen werden *)
          begin
            if deutsch
              then
                writeln (' Schreiben des ', j:4, '. TG-Profils . . .')
              else
                writeln (' writing ', j:4, '. dive profile . . .');
            allgemeine_daten_schreiben;

            (* 19 Bytes "TG-Kopf" *)
            tg_kopfdaten_lesen_schreiben;

            writeln (udcfdat, '    <deltamode/>');
            writeln (udcfdat, '    <samples>');
            writeln (udcfdat, '      <delta>20.0</delta>');
            writeln (udcfdat, '      <switch>air</switch>');
            writeln (udcfdat, '      <d> 0.0</d>');

            (* TG-Profil beschreibende Bytes *)
            i := 0;
            n := 0;
            erste_tiefen_bytes := true;
            while i < anz_tg_bytes - 19 do
              begin
                case i mod 7 of
                  0 : n := 2;
                  3 : n := 1;
                  5 : n := 1
                end;

                lese_1_byte (b [0]);
                lese_1_byte (b [1]);

                if not erste_tiefen_bytes
                  then
                    begin
                      (* Tiefe 10 bit *)
                      tiefe := (b [0] * 4 + (b [1] and 192) div 64) * (160.0 / 1024.0);
                      write (udcfdat, '      <d>', tiefe:4:1, '</d>');

                      (* Alarme *)
                      if b [1] and  2 =  2 then write (udcfdat, '<alarm>rbt</alarm>');
                      if b [1] and  4 =  4 then write (udcfdat, '<alarm>ascent</alarm>');
                      if b [1] and  8 =  8 then write (udcfdat, '<alarm>deco</alarm>');
                      if b [1] and 16 = 16 then write (udcfdat, '<alarm>breath</alarm>');
                    end;

                if n = 2
                  then
                    begin
                      lese_1_byte (b [2]);

                      if not erste_tiefen_bytes
                        then
                          begin
                            if b [2] and 1 = 1 then write (udcfdat, '<work>1</work>')
                            else if b [2] and 2 = 2 then write (udcfdat, '<work>2</work>>')
                            else if b [2] and 4 = 4 then write (udcfdat, '<work>4</work>')
                          end
                    end;
                if not erste_tiefen_bytes then writeln (udcfdat);

                (* Die ersten beiden Tiefen-Bytes sind "Schrott" und *)
                (* enthalten Unsinn. Deshalb mssen sie bersprungen *)
                (* werden und drfen keinesfalls als Tiefendaten in die *)
                (* UDCF-Datei geschrieben werden ! Auch DATATRAK *)
                (* bergeht diese beiden Bytes. *)
                erste_tiefen_bytes := false;

                inc (i, n + 1)
              end;
            writeln (udcfdat, '    </samples>');
            writeln (udcfdat, '  </dive>')
          end
        else
          begin
            if deutsch then writeln (' keine Profildaten vorhanden !')
                       else writeln (' no dive profile data !');
            lese_1_byte (b [0]);
            while b [0] <> 160 do lese_1_byte (b [0]);
            vorheriger_tg_mit_grafik := false
          end
    end; (* tg_daten_lesen_und_schreiben *)

  procedure udcf_dateikopf_schreiben;

    begin
      getdate (year, month, day, dow);
      gettime (hour, minute, sek, hun);
      assign (udcfdat, outputdatei);
      rewrite (udcfdat);
      writeln (udcfdat, '<profile udcf=1>');
      writeln (udcfdat, '<!-- Conversion from Aladin logfile to UDCF by aladin2udcf-', version, ' -->');
      writeln (udcfdat, '<!-- Author : Kai Schr"oder -->');
      writeln (udcfdat, '<!-- Date : ', day, '.', month, '.', year, ', Time : ', hour, ':', minute, ' -->');
      writeln (udcfdat, '<units>metric</units>');
      writeln (udcfdat, '<device>');
      writeln (udcfdat, '  <vendor>Uwatec</vendor>');
      writeln (udcfdat, '  <model>Aladin Air X</model>');
      writeln (udcfdat, '  <version>1</version>');
      writeln (udcfdat, '</device>');
      writeln (udcfdat, '<repgroup>')
    end; (* udcf_dateikopf_schreiben *)

  procedure udcf_datei_schliessen;

    begin
      writeln (udcfdat, '</repgroup>');
      writeln (udcfdat, '</profile>');
      close (udcfdat)
    end; (* udcf_datei_schliessen *)


begin
  (* wenn kein Kommandozeilen-Parameter bergeben wurde deutsche Texte, *)
  (* bei -e englische Texte *)
  if paramcount > 0
    then
      begin
        if paramcount = 1
          then
            begin
              if paramstr (1) = '-e'
                then
                  deutsch := false
                else
                  begin
                    writeln ('      Falscher Kommandozeilen-Parameter ! / Wrong command line parameter !');
                    halt (1)
                  end
            end
          else
            (* mehr als 1 Parameter bergeben, Abbruch *)
            begin
              writeln ('    Zuviele Kommandozeilen-Parameter ! / Too many command line parameters !');
              halt (1)
            end
      end
    else
      deutsch := true;

  clrscr;
  writeln;
  if deutsch
    then
      begin
        writeln (' ':22, 'Aladin --> UDCF Konvertierung v', version);
        writeln (' ':22, '------------------------------------');
        writeln;
        writeln (' Dateinnamen ggfs. mit vollstndiger Pfadangabe eingeben.');
        writeln (' Die Endung ".log" mu nicht, kann aber, angegeben werden.');
        writeln (' Es wird eine UDCF-Datei gleichen Namens mit der Endung ".udc" erzeugt. ');
        writeln (' Es werden nur TGs mit Profildaten bernommen.');
        writeln (' Es erfolgt keine Fehlerprfung !');
        writeln;
        write (' Logbuch-Dateiname : ')
      end
    else
      begin
        writeln (' ':23, 'Aladin --> UDCF Conversion v', version);
        writeln (' ':23, '---------------------------------');
        writeln;
        writeln (' Provide filenames with full path, if necessary.');
        writeln (' The extension ".log" is optional.');
        writeln (' An UDCF file with same name but extension ".udc" is generated. ');
        writeln (' Only dives with profile data are written.');
        writeln (' There is no error checking !');
        writeln;
        write (' Logbook filename : ')
      end;
  readln (dateiname);
  if pos ('.', dateiname) = 0
    then
      begin
        inputdatei := dateiname + '.log';
        outputdatei := dateiname + '.udc'
      end
    else
      begin
        inputdatei := dateiname;
        outputdatei := copy (dateiname, 1, pos ('.', dateiname));
        outputdatei := outputdatei + 'udc'
      end;
  assign (logdat, inputdatei);
  (*$I-*)
  reset (logdat);
  (*$I+*)
  if (ioresult <> 0) or (inputdatei = '')
    then
      begin
        writeln;
        if deutsch
          then
            writeln (' Die Datei existiert nicht ! Bitte die Eingabe berprfen !')
          else
            writeln (' File doesn''t exist ! Please check filename !');
        writeln
      end
    else
      begin
        dateikopf_lesen;
        udcf_dateikopf_schreiben;
        for j := 1 to anz_tg_in_datei do
          begin
            if deutsch then write ('   Lese ', j:4, '. TG --- ')
                       else write ('   reading ', j:4, '. dive --- ');
            allgemeine_daten_lesen;
            tg_daten_lesen_und_schreiben
          end;
        close (logdat);
        udcf_datei_schliessen
      end
end.
